unit DRegCode;

interface

uses Windows, Graphics,
     DGlobalsUnit;

{LSQ}{PURPOSE:  Calculate a Least Squares Fit via routines QR and QR_solve}

{c_eval}{PURPOSE:  Evaluates a polynomial at a given point}
Function Polyevaluation(InPolyDegree:Integer;
                        InPolyArray :PolyArray;
                        InX, InY:Double):Double;

{C_FILE_CORNER}
{PURPOSE: Uses a QR factorization of a polynomial
          to find the file projection corners of an image.
          This information is found from the corners of
          the valid image.}

{c_grderr}
{PURPOSE:  Calculates errors which occured during the gridding process}

{c_steplr}
{PURPOSE:  Computes transformation coefficients based on
           a stepwise regression procedure.}
{int c_steplr
static void corr_coef
static double partial_f
static void calc_coef
static long select_var
static void enter_model
static void exit_model
static void smallest_f
static int f_test
static double icbetaf
static double fraction
static double log_gam }

implementation

{PURPOSE:  Evaluates a polynomial at a given point}
{Determine degree of polynomial
	Return evaluation}
Function Polyevaluation(InPolyDegree:Integer;    // Degree of polynomial
                        InPolyArray :PolyArray;  //Array of polynomial coefficients
                        InX, InY:Double):Double; // X & Y coordinates to be evaluated
Begin
   Polyevaluation:=InX;
{if InPolyDegree = 4 then
return:=(a[0]+ x*(a[1]+ *x*(a[3]+ *x*(a[6]+a[10]* *x+a[11]*
                   *y)+a[7]* *y)+a[4]* *y) + *y*(a[2]+ *y*(a[5]+ *y*(a[9]+
                   a[14]* *y+a[13]* *x)+a[8]* *x+a[12]* *x* *x)))
else
if InPolyDegree = 3 then
return:=(a[0]+ *x*(a[1]+ *x*(a[3]+a[6]* *x+a[7]* *y)+a[4]* *y)
                  + *y*(a[2]+ *y*(a[5]+a[9]* *y+a[8]* *x)))
else
if InPolyDegree = 2 then
return:=(a[0]+ *x*(a[1]+a[3]* *x+a[4]* *y)+ *y*(a[2]+a[5]* *y));}
{return(a[0]+a[1]* *x+a[2]* *y);}
End;


(*
/*****************************************************************************
NAME:				LSQ

PURPOSE:  Calculate a Least Squares Fit via routines QR and QR_solve

VERSION		DATE	AUTHOR
-------		----	------
1.0		8/92	D. Steinwand
1.1		7/94	T. Mittan		Slight additions to 
                                                release as a support.

ALGORITHM Reference:

1.  "Matrix Computations (2nd ed.)", Gene H. Golub & Charles F. Van Loan, John 
    Hopkins University Press, Baltimore 1989 ISBN 0-8018-3739-1
*****************************************************************************/
#include <math.h>
#include "lasmath.h"

static int sign
(
    double val
); 

/* Calculate the QR factorization of the m by n matrix A.  On return, matrix A 
   will contain R in its upper triangle.  The strict lower triangle will 
   contain information about Q.  Vector v is a work vector, but on return will 
   contain additional information about Q.
  ---------------------------------------*/
void c_qrdecomp
(
    double *A,	/* Matrix A (stored by columns) 			*/
    long m,         /* Number of rows in matrix A 				*/
    long n,         /* Number of columns in matrix A 			*/
    double *v,      /* Vector v 						*/
    long flag	/* if True, order matrix A by columns			*/
)
{
long N;                 /* Maximum loop iteration			*/
long k,j;		/* Loop counters 				*/
double *cur_col_A;	/* Pointer to current column of matrix A 	*/
double vtv;		/* temporary variable				*/
long i;
double temp;

if (flag)
   {
   for (i = 0; i < m; i++)
     for (j = i + 1; j < n; j++)
         {
         k = (i * n) + j;
         temp = A[k];
         A[k] = A[(j * n) + i];
         k = (j * n) + i;
         A[k] = temp;
         }
    }

N = ((m-1) > n) ? n : (m-1);
for (k = 0; k < N; k++)
   {
   cur_col_A = A + (k * m);
   house ((m-k), &(cur_col_A[k]), &(v[k]), &vtv, &(cur_col_A[k]));
	
   for (j = k + 1; j < m; j++)
      cur_col_A[j] = v[j];

   row_house(&(A[(k+1)*m+k]), (m-k), (n-k-1), m, &(v[k]), vtv);
   v[k] = vtv;
   }
}

/* Find the least squares solution of Ax = b where A is a full rank m by n 
   matrix with m >= n.  On entry, matrix A should contain the results of the 
   QR routine.  If iflag == 0, then v is a vector of length m containing the 
   results of the QR routine.  If iflag != 0, the entries in v are 
   reconstructed--the vector v is not changed.  The solution will be stored 
   in the first n entries of vector b.
  -----------------------------------*/
void c_qrsolve
(
    double *A,      /* Matrix A (stored by columns) 			*/
    long m,         /* Number of rows in matrix A 				*/
    long n,         /* Number of columns in matrix A 			*/
    double *v,      /* Vector v 						*/
    double *b,	/* Vector b 						*/
    long iflag     /* flag (see comments)					*/
)
{
long N;
long k,j;		/* Loop counters 				*/
double *cur_col_A;      /* Pointer to current column of matrix A 	*/
double sum;		/* Accumulator 					*/
double vtv;		/* Temporary variable				*/
double alpha;		/* Temporary variable				*/

N = ((m-1) > n) ? n : (m-1);
for (k = 0; k < N; k++)
   {
   cur_col_A = A + (k * m);
   sum = b[k];
   for (j = k + 1; j < m; j++)
      sum += (b[j] * cur_col_A[j]);
   if (iflag == 0) vtv = v[k];
   else
      {
      vtv = 1.0;
      for (j = k + 1; j < m; j++)
         vtv += (cur_col_A[j] * cur_col_A[j]);
      }
   alpha = -2.0 / vtv * sum;
   b[k] += alpha;
   for (j= k + 1; j < m; j++)
      b[j] += (alpha * cur_col_A[j]);
   }

/* Backsolve
  ---------*/
b[n-1] /= A[(n-1)*m+n-1];
for (k = n-1; k--;)
   {
   cur_col_A = A + ((k+1) * m);
   for (j = 0; j < k+1; j++)
      b[j] -= (b[k+1] * cur_col_A[j]);
   b[k] /= A[k*m+k];
   }
}

/* Given vector x of length n, find a vector v of length n such that v[0] = 1,
   and for the Householder matrix P defined by v, Px = (Px, 0, ..., 0)
   transpose.
  ----------*/
void house
(
    long n,		/* Input--dimension of vectors x and v 			*/
    double *x,	/* Input--vector x 					*/
    double *v,	/* Output--vector v 					*/
    double *vtv,	/* Output--Product of vector v transpose and v 		*/
    double *Px	/* Output--Product of Householder matrix P and vector x */
)
{
long i;		/* Loop counter 					*/
double u;	/* norm	variable					*/
long sign_x0 = 1;	/* sign variable (contains 1 or -1 ?)			*/
double beta;	/* temporary variable					*/

for (i = 0; i < n; i++)
   v[i] = x[i];
u = norm_2(x, n);
if (u != 0.0)
   {
   sign_x0 = sign(x[0]);
   beta = x[0] + sign_x0 * u;
   for (i = 1; i < n; i++)
      v[i] /= beta;
   *vtv = 2.0 * ((u * u) + (u * fabs(x[0]))) / (beta * beta);
   }
else *vtv = 1.0;
v[0] = 1.0;
*Px = -sign_x0 * u;
}

/* Replace the m by n matrix A by the product PA, where P is the Householder
   Transformation defined by vector v.  vtv should contain the product of
   v transpose v when this routine is called.
  ------------------------------------------*/
void row_house
(
    double *A,	/* Matrix A (stored by columns) 			*/
    long m,		/* Number of rows in matrix A 				*/
    long n,		/* Number of columns in matrix A 			*/
    long mdim,	/* Row dimension 					*/
    double *v,	/* Vector v 						*/
    double vtv	/* Product of vector v transpose and vector v 		*/
)
{
double *cur_col_A;	/* Pointer to current column of matrix A 	*/
double alpha;		/* Temporary variable				*/
long i,k;		/* Loop counters 				*/

for (k = 0; k < n; k++)
   {
   cur_col_A = A + (k * mdim);
   for (alpha = 0.0, i = 0; i < m; i++)
      alpha += v[i] * cur_col_A[i];
   alpha *= -(2.0 / vtv);
   for (i = 0; i < m; i++)
      cur_col_A[i] += (alpha * v[i]);
   }
}
   
/* Function to calculate the 2-norm of a vector
  --------------------------------------------*/
double norm_2
(
    double *x,
    long n
)
{
long i; double sum = 0.0;
for (i = 0; i < n; i++) sum += x[i] * x[i];
return(sqrt(sum)); 
}

/* Function to return the sign of a number
---------------------------------------*/
static int sign
(
    double val
) 
{ 
    if (val < 0.0) return(-1); return(1); 
}

*)

end.
