{*******************************************************}
{                                                       }
{                      Tachyon Unit                     }
{    Vector Raster Geographic Information Synthesis     }
{                     VOICE  ..  Tracer                 }
{                     GRIP ICE .. Tongs                 }
{                Digital Terrain Mapping                }
{               Image Locatable Holographics            }
{                          SOS MAP                      }
{  Surreal Object Synthesis Multimedia Analysis Product }
{                   Fractal3D  Life MOW                 }
{       Copyright (c) 1995,2006  Ivan Lee Herring       }
{                                                       }
{*******************************************************}
unit buzLamaz;

{*******************************************************************************
NAME                  LAMBERT AZIMUTHAL EQUAL-AREA

PURPOSE:	Transforms input longitude and latitude to Easting and
  Northing for the Lambert Azimuthal Equal-Area projection.  The
  longitude and latitude must be in radians.  The Easting
  and Northing values will be returned in meters.

PROGRAMMER              DATE
----------              ----
D. Steinwand, EROS      March, 1991

This function was adapted from the Lambert Azimuthal Equal Area projection
code (FORTRAN) in the General Cartographic Transformation Package software
which is available from the U.S. Geological Survey National Mapping Division.

ALGORITHM REFERENCES

1.  "New Equal-Area Map Projections for Noncircular Regions", John P. Snyder,
    The American Cartographer, Vol 15, No. 4, October 1988, pp. 341-355.

2.  Snyder, John P., "Map Projections--A Working Manual", U.S. Geological
    Survey Professional Paper 1395 (Supersedes USGS Bulletin 1532), United
    State Government Printing Office, Washington D.C., 1987.

3.  "Software Documentation for GCTP General Cartographic Transformation
    Package", U.S. Geological Survey National Mapping Division, May 1982.
*******************************************************************************/
#include <stdio.h>
#include "gctpc/cproj.h"
#include "gctpc/report.h"}
interface
uses Math;
// Variables common to all subroutines in this code file
var
  lon_center, //* Center longitude (projection center)
    lat_center, //* Center latitude (projection center)
    RadiusES, //* Radius of the earth (sphere)
    sin_lat_o, //* Sine of the center latitude
    cos_lat_o, //* Cosine of the center latitude
    false_easting, //* x offset in meters
    false_northing: Double; //* y offset in meters




function
  lamazforint(r, center_long, center_lat, false_east,
  false_north: double): Integer;

function
  lamazfor(lon, lat: double; var x, y: double): Integer;

function
  lamazinvint(r, center_long, center_lat, false_east,
  false_north: double): Integer;

function
  lamazinv(x, y: double; var lon, lat: double): Integer;

implementation

uses buzGlobal;

function
  lamazforint(r, center_long, center_lat, false_east,
  false_north: double): Integer;
{Initialize the Lambert Azimuthal Equal Area projection
long lamazforint(double r, double center_long, double center_lat, double false_east, double false_north)
             /* (I) Radius of the earth (sphere) 	*/
                     /* (I) Center longitude 		*/
                    /* (I) Center latitude 			*/
                    /* x offset in meters			*/
                     /* y offset in meters			*/}
begin
//* Place parameters in static storage for common use
  RadiusES := r;
  lon_center := center_long;
  lat_center := center_lat;
  false_easting := false_east;
  false_northing := false_north;
    sin_lat_o := Sin(center_lat);
    cos_lat_o := Cos(center_lat);
  {sincos(center_lat, sin_lat_o, cos_lat_o);}

//* Report parameters to the user
  ptitle('LAMBERT AZIMUTHAL EQUAL-AREA');
  radius(r);
  cenlon(center_long);
  cenlat(center_lat);
  offsetp(false_easting, false_northing);
  result := (OK);
end;

function
  lamazfor(lon, lat: double; var x, y: double): Integer;
{Lambert Azimuthal Equal Area forward equations--mapping lat,long to x,y
long lamazfor(double lon, double lat, double *x, double *y)
              /* (I) Longitude */
              /* (I) Latitude */
             /* (O) X projection coordinate */
             /* (O) Y projection coordinate */}
var
{double adjust_lon(double x);
/* Function to adjust longitude to -180 - 180 	*/}
  delta_lon, //* Delta longitude (Given longitude - center 	*/
    sin_delta_lon, //* Sine of the delta longitude 			*/
    cos_delta_lon, //* Cosine of the delta longitude 		*/
    sin_lat, //* Sine of the given latitude 			*/
    cos_lat, //* Cosine of the given latitude 		*/
    g, //* temporary varialbe				*/
    ksp: double; //* heigth above elipsiod			*/
{char mess[60];}
begin
//* Forward equations

  delta_lon := AdjustLon(lon - lon_center);
    sin_lat := Sin(lat);
    cos_lat := Cos(lat);
  {sincos(lat, sin_lat, cos_lat);}
    sin_delta_lon := Sin(delta_lon);
    cos_delta_lon := Cos(delta_lon);
  {sincos(delta_lon, sin_delta_lon, cos_delta_lon);}
  g := sin_lat_o * sin_lat + cos_lat_o * cos_lat * cos_delta_lon;
  if (g = -1.0) then
  begin
{   sprintf(mess, "Point projects to a circle of radius = %lf\n", 2.0 * R);}
    {PError('mess', 'lamaz-forward');}
    DoProjError(113,113,0,0,g);
    result := (113);
    exit;
  end;
  ksp := RadiusES * sqrt(2.0 / (1.0 + g));
  x := ksp * cos_lat * sin_delta_lon + false_easting;
  y := ksp * (cos_lat_o * sin_lat - sin_lat_o * cos_lat *
    cos_delta_lon) +
    false_northing;
  result := (OK);
end;


function
  lamazinvint(r, center_long, center_lat, false_east,
  false_north: double): Integer;
{Initialize the Lambert Azimuthal Equal Area projection
long lamazinvint(double r, double center_long, double center_lat,
 double false_east, double false_north)
             /* (I) Radius of the earth (sphere)
                     /* (I) Center longitude
                    /* (I) Center latitude
                    /* x offset in meters
                     /* y offset in meters}
begin
//* Place parameters in static storage for common use
  RadiusES := r;
  lon_center := center_long;
  lat_center := center_lat;
  false_easting := false_east;
  false_northing := false_north;
    sin_lat_o := Sin(center_lat);
    cos_lat_o := Cos(center_lat);
  {sincos(center_lat, sin_lat_o, cos_lat_o);}

//* Report parameters to the user

  ptitle('LAMBERT AZIMUTHAL EQUAL-AREA');
  radius(r);
  cenlon(center_long);
  cenlat(center_lat);
  offsetp(false_easting, false_northing);
  result := (OK);
end;


function
  lamazinv(x, y: double; var lon, lat: double): Integer;
{Lambert Azimuthal Equal Area inverse equations--mapping x,y to lat,long
long lamazinv(double x, double y, double *lon, double *lat)
           /* (I) X projection coordinate */
           /* (I) Y projection coordinate */
              /* (O) Longitude */
              /* (O) Latitude */}
{double adjust_lon(double x);
 /* Function to adjust longitude to -180 - 180 */}
{double asinz(double con);}
var
  Rh,
    z, //* Great circle dist from proj center to given point */
    sin_z, //* Sine of z */
    cos_z, //* Cosine of z */
    temp: double; //* Re-used temporary variable */
begin
//* Inverse equations
  x := x - false_easting;
  y := y - false_northing;
  Rh := sqrt(x * x + y * y);
  temp := Rh / (2.0 * RadiusES);
  if (temp > 1) then
  begin
    {PError('Input data error', 'lamaz-inverse');  }
    DoProjError(115,115,0,0,0);
    result := (115);
    exit;
  end;
  z := 2.0 * asinz(temp);
    sin_z := Sin(z);
    cos_z := Cos(z);
  {sincos(z, sin_z, cos_z);}
  lon := lon_center;
  if (abs(Rh) > EPSLN) then
  begin
    lat := asinz(sin_lat_o * cos_z + cos_lat_o * sin_z * y / Rh);
    temp := abs(lat_center) - HALF_PI;
    if (abs(temp) > EPSLN) then
    begin
      temp := cos_z - sin_lat_o * sin(lat);
      if (temp <> 0.0) then
        lon := AdjustLon(lon_center + arctan(x * sin_z * cos_lat_o /
          temp * Rh));
    end
    else if (lat_center < 0.0) then
      lon := AdjustLon(lon_center - arctan(-x / y))
    else lon := AdjustLon(lon_center + arctan(x / -y));
  end
  else lat := lat_center;
  result := (OK);
end;



end.
