{*******************************************************}
{                                                       }
{                      Tachyon Unit                     }
{    Vector Raster Geographic Information Synthesis     }
{                     VOICE  ..  Tracer                 }
{                     GRIP ICE .. Tongs                 }
{                Digital Terrain Mapping                }
{               Image Locatable Holographics            }
{                          SOS MAP                      }
{  Surreal Object Synthesis Multimedia Analysis Product }
{                   Fractal3D  Life MOW                 }
{       Copyright (c) 1995,2006  Ivan Lee Herring       }
{                                                       }
{*******************************************************}
unit buzLamcc;

{***********************************************************************
NAME                            LAMBERT CONFORMAL CONIC
PURPOSE:	Transforms input longitude and latitude to Easting and
  Northing for the Lambert Conformal Conic projection.  The
  longitude and latitude must be in radians.  The Easting
  and Northing values will be returned in meters.
PROGRAMMER              DATE
----------              ----
T. Mittan               2-26-93
ALGORITHM REFERENCES

1.  Snyder, John P., "Map Projections--A Working Manual", U.S. Geological
    Survey Professional Paper 1395 (Supersedes USGS Bulletin 1532), United
    State Government Printing Office, Washington D.C., 1987.

2.  Snyder, John P. and Voxland, Philip M., "An Album of Map Projections",
    U.S. Geological Survey Professional Paper 1453 , United State Government
*******************************************************************************/
#include "gctpc/cproj.h"
#include "gctpc/report.h"}
interface
uses Math;
//* Variables common to all subroutines in this code file
var
  r_major, //* major axis                   */
    r_minor, //* minor axis                   */
    es, //* eccentricity squared         */
    e, //* eccentricity                 */
    center_lon, //* center longituted            */
    center_lat, //* cetner latitude              */
    ns, //* ratio of angle between meridian*/
    f0, //* flattening of ellipsoid      */
    rh, //* height above ellipsoid       */
    false_easting, //* x offset in meters           */
    false_northing: Double; //* y offset in meters           */




function
  lamccforint(r_maj, r_min, lat1, lat2, c_lon,
  c_lat, false_east, false_north: double): Integer;

function
  lamccfor(lon, lat: double; var x, y: double): Integer;

function
  lamccinvint(r_maj, r_min, lat1, lat2, c_lon,
  c_lat, false_east, false_north: double): Integer;

function
  lamccinv(x, y: double; var lon, lat: double): Integer;

implementation

uses buzGlobal;

function
  lamccforint(r_maj, r_min, lat1, lat2, c_lon,
  c_lat, false_east, false_north: double): Integer;
{Initialize the Lambert Conformal conic projection
long
lamccforint(double r_maj, double r_min,
double lat1, double lat2,
double c_lon, double c_lat,
double false_east, double false_north)
                                /* center latitude
                                /* center longitude
                                /* first standard parallel
                                /* second standard parallel
                                /* major axis
                                /* minor axis
                                /* x offset in meters
                                /* y offset in meters}

var
  sin_po, ///* sin value                            */
    cos_po, ///* cos value                            */
    con, //* temporary variable                   */
    ms1, //* small m 1                            */
    ms2, //* small m 2                            */
    temp, //* temporary variable                   */
    ts0, //* small t 0                            */
    ts1, //* small t 1                            */
    ts2: double; //* small t 2                            */
{double msfnz(double eccent, double sinphi, double cosphi);
 /* function to compute small m          */
double tsfnz(double eccent, double phi, double sinphi);
 /* function to compute small t          */}
begin
  r_major := r_maj;
  r_minor := r_min;
  false_northing := false_north;
  false_easting := false_east;

//* Standard Parallels cannot be equal and on opposite sides of the equator
  if (abs(lat1 + lat2) < EPSLN) then
  begin
    {PError('Equal Latitiudes for St. Parallels on opposite sides of equator',
      'lamcc-for');}
    DoProjError(41,41,0,0,0);
    result := (41);
    exit;
  end;
  temp := r_minor / r_major;
  es := 1.0 - SQR(temp);
  e := sqrt(es);

  center_lon := c_lon;
  center_lat := c_lat;
    sin_po := Sin(lat1);
    cos_po := Cos(lat1);
  {sincos(lat1, sin_po, cos_po);}
  con := sin_po;
  ms1 := msfnz(e, sin_po, cos_po);
  ts1 := tsfnz(e, lat1, sin_po);
    sin_po := Sin(lat2);
    cos_po := Cos(lat2);
  {sincos(lat2, sin_po, cos_po);}
  ms2 := msfnz(e, sin_po, cos_po);
  ts2 := tsfnz(e, lat2, sin_po);
  sin_po := sin(center_lat);
  ts0 := tsfnz(e, center_lat, sin_po);

  if (abs(lat1 - lat2) > EPSLN) then
    ns := ln(ms1 / ms2) / ln(ts1 / ts2)
  else ns := con;
  f0 := ms1 / (ns * power(ts1, ns));
  rh := r_major * f0 * power(ts0, ns);

//* Report parameters to the user
  ptitle('LAMBERT CONFORMAL CONIC');
  radius2(r_major, r_minor);
  stanparl(lat1, lat2);
  cenlonmer(center_lon);
  origin(c_lat);
  offsetp(false_easting, false_northing);

  result := (OK);
end;


function
  lamccfor(lon, lat: double; var x, y: double): Integer;
{Lambert Conformal conic forward equations--mapping lat,long to x,y
long
lamccfor(double lon, double lat, double *x, double *y)
                                /* (I) Longitude                */
                                /* (I) Latitude                 */
                                /* (O) X projection coordinate  */
                                /* (O) Y projection coordinate}

var
  con, //* temporary angle variable             */
    rh1, //* height above ellipsoid               */
    sinphi, //* sin value                            */
    theta, //* angle                                */
    ts: double; //* small value t                        */
{double tsfnz(double eccent, double phi, double sinphi);
/* function to compute small t          */
double adjust_lon(double x);
/* function to adust the longitude      */}
begin
  con := abs(abs(lat) - HALF_PI);
  if (con > EPSLN) then
  begin
    sinphi := sin(lat);
    ts := tsfnz(e, lat, sinphi);
    rh1 := r_major * f0 * power(ts, ns);
  end
  else
  begin
    con := lat * ns;
    if (con <= 0) then
    begin
      {PError('Point can not be projected', 'lamcc-for');}
      DoProjError(41,44,0,0,0);
      result := (44);
      exit;
    end;
    rh1 := 0;
  end;
  theta := ns * AdjustLon(lon - center_lon);
  x := rh1 * sin(theta) + false_easting;
  y := rh - rh1 * cos(theta) + false_northing;

  result := (OK);
end;




function
  lamccinvint(r_maj, r_min, lat1, lat2, c_lon,
  c_lat, false_east, false_north: double): Integer;
{Initialize the Lambert Conformal Conic projection
long
lamccinvint(double r_maj, double r_min,
double lat1, double lat2, double c_lon,
double c_lat, double false_east, double false_north)
                 /* center latitude
                 /* center longitude
                /* first standard parallel
                /* second standard parallel
                 /* major axis
                 /* minor axis
                     /* x offset in meters
                      /* y offset in meters}
var
  sin_po, //* sin value			*/
    cos_po, //* cos value			*/
    con, //* temporary sin value		*/
    ms1, //* small m 1			*/
    ms2, //* small m 2			*/,
    temp, //* temporary variable		*/
    ts0, //* small t 0			*/
    ts1, //* small t 1			*/
    ts2: double; //* small t 2			*/
{double msfnz(double eccent, double sinphi, double cosphi);
    /* function to compute small m	*/
double tsfnz(double eccent, double phi, double sinphi);
   /* function to compute small t	*/}
begin
  r_major := r_maj;
  r_minor := r_min;
  false_easting := false_east;
  false_northing := false_north;

//* Standard Parallels cannot be equal and on opposite sides of the equator
  if (abs(lat1 + lat2) < EPSLN) then
  begin
   {PError('Equal Latitiudes for St. Parallels on opposite sides of equator',
      'lamcc-inv');}
    DoProjError(42,41,0,0,0);
    result := (41);
    exit;
  end;
  temp := r_minor / r_major;
  es := 1.0 - SQR(temp);
  e := sqrt(es);

  center_lon := c_lon;
  center_lat := c_lat;
    sin_po := Sin(lat1);
    cos_po := Cos(lat1);
  {sincos(lat1, sin_po, cos_po);}
  con := sin_po;
  ms1 := msfnz(e, sin_po, cos_po);
  ts1 := tsfnz(e, lat1, sin_po);
    sin_po := Sin(lat2);
    cos_po := Cos(lat2);
  {sincos(lat2, sin_po, cos_po);}
  ms2 := msfnz(e, sin_po, cos_po);
  ts2 := tsfnz(e, lat2, sin_po);
  sin_po := sin(center_lat);
  ts0 := tsfnz(e, center_lat, sin_po);

  if (abs(lat1 - lat2) > EPSLN) then
    ns := ln(ms1 / ms2) / ln(ts1 / ts2)
  else
    ns := con;
  f0 := ms1 / (ns * power(ts1, ns));
  rh := r_major * f0 * power(ts0, ns);



//* Report parameters to the user
  ptitle('LAMBERT CONFORMAL CONIC');
  radius2(r_major, r_minor);
  stanparl(lat1, lat2);
  cenlonmer(center_lon);
  origin(c_lat);
  offsetp(false_easting, false_northing);

  result := (OK);
end;


function
  lamccinv(x, y: double; var lon, lat: double): Integer;
{ Lambert Conformal Conic inverse equations--mapping x,y to lat/long
  -----------------------------------------------------------------*/
long
lamccinv(double x, double y, double *lon, double *lat)
            /* (O) X projection coordinate 	*/
            /* (O) Y projection coordinate 	*/
               /* (I) Longitude 		*/
               /* (I) Latitude}

var
  rh1, //* height above ellipsoid
    con, //* sign variable
    ts, //* small t
    theta: Double; //* angle
{double phi2z(double eccent, double ts, long int *flag);
  /* function to compute phi	*/
double adjust_lon(double x);
  /* functio to adjust longitude }
  flag: Integer; //* error flag
begin
  flag := 0;
  x := x - false_easting;
  y := rh - y + false_northing;
  if (ns > 0) then
  begin
    rh1 := sqrt(x * x + y * y);
    con := 1.0;
  end
  else
  begin
    rh1 := -sqrt(x * x + y * y);
    con := -1.0;
  end;
  theta := 0.0;
  if (rh1 <> 0) then
    theta := arctan((con * x) / (con * y));
  if ((rh1 <> 0) or (ns > 0.0)) then
  begin
    con := 1.0 / ns;
    ts := power((rh1 / (r_major * f0)), con);
    lat := phi2z(e, ts, flag);
    if (flag <> 0) then begin result := (flag); exit end;
  end
  else lat := -HALF_PI;
  lon := AdjustLon(theta / ns + center_lon);
  result := (OK);
end;

end.
