{*******************************************************}
{                                                       }
{                      Tachyon Unit                     }
{    Vector Raster Geographic Information Synthesis     }
{                     VOICE  ..  Tracer                 }
{                     GRIP ICE .. Tongs                 }
{                Digital Terrain Mapping                }
{               Image Locatable Holographics            }
{                          SOS MAP                      }
{  Surreal Object Synthesis Multimedia Analysis Product }
{                   Fractal3D  Life MOW                 }
{       Copyright (c) 1995,2006  Ivan Lee Herring       }
{                                                       }
{*******************************************************}
unit buzOrth;

{********************************************************************
NAME                             ORTHOGRAPHIC
PURPOSE:	Transforms input longitude and latitude to Easting and
  Northing for the Orthographic projection.  The
  longitude and latitude must be in radians.  The Easting
  and Northing values will be returned in meters.
PROGRAMMER              DATE
----------              ----
T. Mittan		Mar, 1993
ALGORITHM REFERENCES
1.  Snyder, John P., "Map Projections--A Working Manual", U.S. Geological
    Survey Professional Paper 1395 (Supersedes USGS Bulletin 1532), United
    State Government Printing Office, Washington D.C., 1987.

2.  Snyder, John P. and Voxland, Philip M., "An Album of Map Projections",
    U.S. Geological Survey Professional Paper 1453 , United State Government
    Printing Office, Washington D.C., 1989.
*******************************************************************************/
#include "gctpc/cproj.h"
#include "gctpc/report.h"}
interface
uses Math;
//* Variables common to all subroutines in this code file
var
  r_major, //* major axis 				*/
    lon_center, //* Center longitude (projection center) */
    lat_origin, //* center latitude			*/
    false_northing, //* y offset in meters			*/
    false_easting, //* x offset in meters			*/
    sin_p14, //* sin of center latitude		*/
    cos_p14: Double; //* cos of center latitude		*/


function
  orthforint(r_maj, center_lon, center_lat, false_east,
  false_north: double): Integer;

function
  orthfor(lon, lat: double; var x, y: double): Integer;

function
  orthinvint(r_maj, center_lon, center_lat,
  false_east, false_north: double): Integer;

function
  orthinv(x, y: double; var lon, lat: double): Integer;

implementation
uses buzGlobal;

function
  orthforint(r_maj, center_lon, center_lat, false_east,
  false_north: double): Integer;
{Initialize the Orthographic projection
long orthforint(double r_maj, double center_lon,
double center_lat, double false_east, double false_north)
                /* major axis
                    /* center longitude
                    /* center latitude
                    /* x offset in meters
                     /* y offset in meters}
  {var temp:double;}//* temporary variable
begin
//* Place parameters in static storage for common use
  r_major := r_maj;
  lon_center := center_lon;
  lat_origin := center_lat;
  false_northing := false_north;
  false_easting := false_east;
    sin_p14 := Sin(center_lat);
    cos_p14 := Cos(center_lat);
{  sincos(center_lat, sin_p14, cos_p14);}

//* Report parameters to the user

  ptitle('ORTHOGRAPHIC');
  radius(r_major);
  cenlonmer(lon_center);
  origin(lat_origin);
  offsetp(false_easting, false_northing);
  result := (OK);
end;


function
  orthfor(lon, lat: double; var x, y: double): Integer;
{Orthographic forward equations--mapping lat,long to x,y
long orthfor(double lon, double lat, double *x, double *y)
              /* (I) Longitude 		*/
              /* (I) Latitude 		*/
             /* (O) X projection coordinate 	*/
             /* (O) Y projection coordinate 	*/}
var
{double adjust_lon(double x);
/* Function to adjust longitude to -180 - 180 	*/}
  sinphi, cosphi, //* sin and cos value				*/
    dlon, //* delta longitude value			*/
    coslon, //* cos of longitude				*/
    ksp, //* scale factor					*/
    g: double;
begin
//* Forward equations

  dlon := AdjustLon(lon - lon_center);
    sinphi := Sin(lat);
    cosphi := Cos(lat);
  {sincos(lat, sinphi, cosphi);}
  coslon := cos(dlon);
  g := sin_p14 * sinphi + cos_p14 * cosphi * coslon;
  ksp := 1.0;
  if ((g > 0) or (abs(g) <= EPSLN)) then
  begin
    x := false_easting + r_major * ksp * cosphi * sin(dlon);
    y := false_northing + r_major * ksp * (cos_p14 * sinphi -
      sin_p14 * cosphi * coslon);
  end
  else
  begin
    {PError('Point can not be projected', 'orth-for');  }
    DoProjError(143,143,0,0,0);
    result := (143);
    exit;
  end;
  result := (OK);
end;



function
  orthinvint(r_maj, center_lon, center_lat,
  false_east, false_north: double): Integer;
{Initialize the Orthographic projection
long orthinvint(double r_maj, double center_lon,
double center_lat, double false_east, double false_north)
                /* major axis
                    /* center longitude
                    /* center latitude
                    /* x offset in meters
                     /* y offset in meters}
begin
//* Place parameters in static storage for common use

  r_major := r_maj;
  lon_center := center_lon;
  lat_origin := center_lat;
  false_northing := false_north;
  false_easting := false_east;
    sin_p14 := Sin(center_lat);
    cos_p14 := Cos(center_lat);
  {sincos(center_lat, sin_p14, cos_p14);}

//* Report parameters to the user

  ptitle('ORTHOGRAPHIC');
  radius(r_major);
  cenlonmer(lon_center);
  origin(lat_origin);
  offsetp(false_easting, false_northing);
  result := (OK);
end;

function
  orthinv(x, y: double; var lon, lat: double): Integer;
{long
orthinv(double x, double y, double *lon, double *lat)
            /* (O) X projection coordinate 	*/
            /* (O) Y projection coordinate 	*/
               /* (I) Longitude 		*/
               /* (I) Latitude 		*/}
var
{double adjust_lon(double x);
/* Function to adjust longitude to -180 - 180 	*/
double asinz(double con);}
  rh, //* height above ellipsoid			*/
    z, //* angle					*/
    sinz, cosz, //* sin of z and cos of z			*/
{ temp,   }
  con: double;
begin
//* Inverse equations

  x := x - false_easting;
  y := y - false_northing;
  rh := sqrt(x * x + y * y);
  if (rh > r_major + 0.0000001) then
  begin
    {PError('Input data error', 'orth-inv');}
    DoProjError(145,145,0,0,0);
    result := (145);
    exit;
  end;
  z := asinz(rh / r_major);
    sinz := Sin(z);
    cosz := Cos(z);
  {sincos(z, sinz, cosz);}
  lon := lon_center;
  if (abs(rh) <= EPSLN) then
  begin
    lat := lat_origin;
    result := (OK);
    exit;
  end;
  lat := asinz(cosz * sin_p14 + (y * sinz * cos_p14) / rh);
  con := abs(lat_origin) - HALF_PI;
  if (abs(con) <= EPSLN) then
  begin
    if (lat_origin >= 0) then
    begin
      lon := AdjustLon(lon_center + arctan(x / -y));
      result := (OK);
      exit;
    end
    else
    begin
      lon := AdjustLon(lon_center - arctan(-x / y));
      result := (OK);
      exit;
    end;
  end;
  con := cosz - sin_p14 * sin(lat);
  if ((abs(con) >= EPSLN) or (abs(x) >= EPSLN)) then
    lon := AdjustLon(lon_center + arctan((x * sinz * cos_p14) / (con
      * rh)));

  result := (OK);
end;



end.
