{*******************************************************}
{                                                       }
{                      Tachyon Unit                     }
{    Vector Raster Geographic Information Synthesis     }
{                     VOICE  ..  Tracer                 }
{                     GRIP ICE .. Tongs                 }
{                Digital Terrain Mapping                }
{               Image Locatable Holographics            }
{                          SOS MAP                      }
{  Surreal Object Synthesis Multimedia Analysis Product }
{                   Fractal3D  Life MOW                 }
{       Copyright (c) 1995,2006  Ivan Lee Herring       }
{                                                       }
{*******************************************************}
unit buzPs;

{**********************************************************************
NAME                            POLAR STEREOGRAPHIC
PURPOSE:	Transforms input longitude and latitude to Easting and
  Northing for the Polar Stereographic projection.  The
  longitude and latitude must be in radians.  The Easting
  and Northing values will be returned in meters.
PROGRAMMER              DATE
----------              ----
T. Mittan		2-26-93
ALGORITHM REFERENCES
1.  Snyder, John P., "Map Projections--A Working Manual", U.S. Geological
    Survey Professional Paper 1395 (Supersedes USGS Bulletin 1532), United
    State Government Printing Office, Washington D.C., 1987.
2.  Snyder, John P. and Voxland, Philip M., "An Album of Map Projections",
    U.S. Geological Survey Professional Paper 1453 , United State Government
    Printing Office, Washington D.C., 1989.
#include "gctpc/cproj.h"
#include "gctpc/report.h"}
interface
uses Math;
//* Variables common to all subroutines in this code file
var
  r_major, //* major axis
    r_minor, //* minor axis
    es, //* eccentricity squared
    e, //* eccentricity
    e4, //* e4 calculated from eccentricity
    center_lon, //* center longitude
    center_lat, //* center latitude
    fac, //* sign variable
    ind, //* flag variable
    mcs, //* small m
    tcs, //* small t
    false_northing, //* y offset in meters
    false_easting: Double; //* x offset in meters



function
  psforint(r_maj, r_min, c_lon, c_lat,
  false_east, false_north: double): Integer;

function
  psfor(lon, lat: double; var x, y: double): Integer;

function
  psinvint(r_maj, r_min, c_lon, c_lat, false_east,
  false_north: double): Integer;

function
  psinv(x, y: double; var lon, lat: double): Integer;

implementation
uses buzGlobal;


function
  psforint(r_maj, r_min, c_lon, c_lat,
  false_east, false_north: double): Integer;
{Initialize the Polar Stereographic projection
long psforint(double r_maj, double r_min,
double c_lon, double c_lat, double false_east, double false_north)
                 /* center longitude		*/
                 /* center latitude		*/
                 /* major axis			*/
                 /* minor axis			*/
                     /* x offset in meters		*/
                      /* y offset in meters}

var
  temp, //* temporary variable		*/
    con1, //* temporary angle		*/
    sinphi, //* sin value			*/
    cosphi: double; //* cos value			*/
{double e4fn(double x);
  /* function to compute e4	*/
double msfnz(double eccent, double sinphi, double cosphi);
    /* function to compute m	*/
double tsfnz(double eccent, double phi, double sinphi);
   /* function to compute t	*/}
begin
  r_major := r_maj;
  r_minor := r_min;
  false_northing := false_north;
  false_easting := false_east;
  temp := r_minor / r_major;
  es := 1.0 - SQR(temp);
  e := sqrt(es);
  e4 := e4fn(e);
  center_lon := c_lon;
  center_lat := c_lat;

  if (c_lat < 0) then fac := -1.0
  else fac := 1.0;
  ind := 0;
  if (abs(abs(c_lat) - HALF_PI) > EPSLN) then
  begin
    ind := 1;
    con1 := fac * center_lat;
    sinphi := Sin(con1);
    cosphi := Cos(con1);
    {sincos(con1, sinphi, cosphi);}
    mcs := msfnz(e, sinphi, cosphi);
    tcs := tsfnz(e, con1, sinphi);
  end;
//* Report parameters to the user

  ptitle('POLAR STEREOGRAPHIC');
  radius2(r_major, r_minor);
  cenlon(center_lon);
  offsetp(false_east, false_north);

  result := (OK);
end;



function
  psfor(lon, lat: double; var x, y: double): Integer;
{Polar Stereographic forward equations--mapping lat,long to x,y
long
psfor(double lon, double lat, double *x, double *y)
              /* (I) Longitude 		*/
              /* (I) Latitude 		*/
             /* (O) X projection coordinate 	*/
             /* (O) Y projection coordinate 	*/}


var
{double adjust_lon(double x);	/* function to adjust longitude	*/
double tsfnz(double eccent, double phi, double sinphi);
  /* function to compute t	*/}
  rh, //* height above ellipsoid
    sinphi, //* sin value
    ts, //* value of small t
    con1, //* adjusted longitude
    con2: Double; //* adjusted latitude
begin
  con1 := fac * AdjustLon(lon - center_lon);
  con2 := fac * lat;
  sinphi := sin(con2);
  ts := tsfnz(e, con2, sinphi);
  if (ind <> 0) then
    rh := r_major * mcs * ts / tcs
  else
    rh := 2.0 * r_major * ts / e4;
  x := fac * rh * sin(con1) + false_easting;
  y := -fac * rh * cos(con1) + false_northing; ;

  result := (OK);
end;


function
  psinvint(r_maj, r_min, c_lon, c_lat, false_east,
  false_north: double): Integer;
{Initialize the Polar Stereographic projection
  --------------------------------------------*/
long
psinvint(double r_maj, double r_min, double c_lon, double c_lat, double false_east, double false_north)

                 /* center longitude		*/
                 /* center latitude		*/
                 /* major axis			*/
                 /* minor axis			*/
                     /* x offset in meters		*/
                      /* y offset in meters}

var
  temp, //* temporary variable		*/
    con1, //* temporary angle		*/
    sinphi, //* sin value			*/
    cosphi, //* cos value			*/
    es: double; //* eccentricity squared         */
{double e4fn(double x);	      	/* function to compute e4	*/
double msfnz(double eccent, double sinphi, double cosphi);
    /* function to compute small m	*/
double tsfnz(double eccent, double phi, double sinphi);
    /* function to compute small t	*/ }
begin
  r_major := r_maj;
  r_minor := r_min;
  false_easting := false_east;
  false_northing := false_north;
  temp := r_minor / r_major;
  es := 1.0 - SQR(temp);
  e := sqrt(es);
  e4 := e4fn(e);
  center_lon := c_lon;
  center_lat := c_lat;

  if (c_lat < 0) then fac := -1.0
  else fac := 1.0;
  ind := 0;
  if (abs(abs(c_lat) - HALF_PI) > EPSLN) then
  begin
    ind := 1;
    con1 := fac * center_lat;
    sinphi := Sin(con1);
    cosphi := Cos(con1);
    {sincos(con1, sinphi, cosphi);}
    mcs := msfnz(e, sinphi, cosphi);
    tcs := tsfnz(e, con1, sinphi);
  end;
//* Report parameters to the user
  ptitle('POLAR STEREOGRAPHIC');
  radius2(r_major, r_minor);
  cenlon(center_lon);
  offsetp(false_east, false_north);

  result := (OK);
end;


function
  psinv(x, y: double; var lon, lat: double): Integer;
{ Polar Stereographic inverse equations--mapping x,y to lat/long
  --------------------------------------------------------------*/
long
psinv(double x, double y, double *lon, double *lat)
            /* (O) X projection coordinate 	*/
            /* (O) Y projection coordinate 	*/
               /* (I) Longitude 		*/
               /* (I) Latitude}
{double phi2z(double eccent, double ts, long int *flag);
   /* function to compute phi	*/
double adjust_lon(double x);
  /* function to adjust longitude	*/}
var
  rh, //* height above ellipsiod
    ts, //* small value t
    temp: double; //* temporary variable
  flag: Integer; //* error flag
begin
  flag := 0;
  x := (x - false_easting) * fac;
  y := (y - false_northing) * fac;
  rh := sqrt(x * x + y * y);
  if (ind <> 0) then
    ts := rh * tcs / (r_major * mcs)
  else
    ts := rh * e4 / (r_major * 2.0);
  lat := fac * phi2z(e, ts, flag);
  if (flag <> 0) then begin result := (flag); exit; end;
  if (rh = 0) then lon := fac * center_lon
  else
  begin
    temp := arctan(x / -y);
    lon := AdjustLon(fac * temp + center_lon);
  end;

  result := (OK);
end;



end.
