{*******************************************************}
{                                                       }
{                      Tachyon Unit                     }
{    Vector Raster Geographic Information Synthesis     }
{                     VOICE  ..  Tracer                 }
{                     GRIP ICE .. Tongs                 }
{                Digital Terrain Mapping                }
{               Image Locatable Holographics            }
{                          SOS MAP                      }
{  Surreal Object Synthesis Multimedia Analysis Product }
{                   Fractal3D  Life MOW                 }
{       Copyright (c) 1995,2006  Ivan Lee Herring       }
{                                                       }
{*******************************************************}
unit buzWvii;

{NAME                  		WAGNER VII

PURPOSE:	Transforms input longitude and latitude to Easting and
  Northing for the Wagner VII projection.  The
  longitude and latitude must be in radians.  The Easting
  and Northing values will be returned in meters.

PROGRAMMER              DATE
----------              ----
D. Steinwand, EROS      May, 1991

This function was implemented with formulas supplied by John P. Snyder.

ALGORITHM REFERENCES

1.  Snyder, John P. and Voxland, Philip M., "An Album of Map Projections",
    U.S. Geological Survey Professional Paper 1453 , United State Government
    Printing Office, Washington D.C., 1989.

2.  Snyder, John P., Personal correspondence, January 1991.
#include "gctpc/cproj.h"
#include "gctpc/report.h"}
interface

uses Math;
// Variables common to all subroutines in this code file
var
  lon_center, //* Center longitude (projection center) */
    R, //* Radius of the earth (sphere) */
    false_easting, //* x offset				*/
    false_northing: Double; //* y offset				*/

function
  wviiforint(r, center_long, false_east, false_north: double):
    Integer;

function
  wviifor(lon, lat: double; var x, y: double): Integer;

function
  wviiinvint(r, center_long, false_east, false_north: double):
    Integer;

function
  wviiinv(x, y: double; var lon, lat: double): Integer;

implementation
uses buzGlobal;


function
  wviiforint(r, center_long, false_east, false_north: double):
    Integer;
{Initialize the Wagner VII projection
long
wviiforint(double r, double center_long, double false_east, double false_north)
             /* (I) Radius of the earth (sphere) */
                     /* (I) Center longitude */
                    /* x offset
                     /* y offset}
begin
//* Place parameters in static storage for common use

  R := r;
  lon_center := center_long;
  false_easting := false_east;
  false_northing := false_north;

//* Report parameters to the user

  ptitle('WAGNER VII');
  radius(r);
  cenlon(center_long);
  offsetp(false_easting, false_northing);
  result := (OK);
end;


function
  wviifor(lon, lat: double; var x, y: double): Integer;
{Wagner VII forward equations--mapping lat,long to x,y
long
wviifor(double lon, double lat, double *x, double *y)
              /* (I) Longitude */
              /* (I) Latitude */
             /* (O) X projection coordinate */
             /* (O) Y projection coordinate */}
var
{double AdjustLon(double x);
 /* Function to adjust longitude to -180 - 180 */}
  delta_lon, //* Delta longitude (Given longitude - center */
    sin_lon, cos_lon,
    s, c0, c1: double;
begin
//* Forward equations

  delta_lon := AdjustLon(lon - lon_center);
    sin_lon := Sin((delta_lon / 3.0));
    cos_lon := Cos((delta_lon / 3.0));
  {sincos((delta_lon / 3.0), sin_lon, cos_lon);}
  s := 0.90631 * sin(lat);
  c0 := sqrt(1 - s * s);
  c1 := sqrt(2.0 / (1.0 + c0 * cos_lon));
  x := 2.66723 * R * c0 * c1 * sin_lon + false_easting;
  y := 1.24104 * R * s * c1 + false_northing;
  result := (OK);
end;

function
  wviiinvint(r, center_long, false_east, false_north: double):
    Integer;
{Initialize the Wagner VII projection
long
wviiinvint(double r, double center_long, double false_east, double false_north)
             /* (I) Radius of the earth (sphere) */
                     /* (I) Center longitude */
                        /* x offset
                        /* y offset}
begin
//* Place parameters in static storage for common use

  R := r;
  lon_center := center_long;
  false_easting := false_east;
  false_northing := false_north;

//* Report parameters to the user

  ptitle('WAGNER VII');
  radius(r);
  cenlon(center_long);
  offsetp(false_easting, false_northing);
  result := (OK);
end;

function
  wviiinv(x, y: double; var lon, lat: double): Integer;
{Wagner VII inverse equations--mapping x,y to lat,long

long
wviiinv(double x, double y, double *lon, double *lat)

           /* (I) X projection coordinate */
           /* (I) Y projection coordinate */
              /* (O) Longitude */
              /* (O) Latitude */}
var
{double AdjustLon(double x);
/* Function to adjust longitude to -180 - 180 */
double asinz(double con);
/* Function to ensure arcsin is valid */ }
{ temp,	}//* Re-used temporary variable */
  t1, t2, p, c: double;
begin
//* Inverse equations

  x := x - false_easting;
  y := y - false_northing;
  t1 := x / 2.66723;
  t2 := y / 1.24104;
  t1 := t1 * t1;
  t2 := t2 * t2;
  p := sqrt(t1 + t2);
  c := 2.0 * asinz(p / (2.0 * R));
  lat := asinz(y * sin(c) / (1.24104 * 0.90631 * p));
  lon := AdjustLon(lon_center + 3.0 * arctan(x * tan(c) / 2.66723 *
    p));
  result := (OK);
end;



end.
